import csv
import re
from glob import glob

from cutils import ProgressBar
from nltk.corpus import stopwords
from vaderSentiment.vaderSentiment import sentiment as vaderSentiment
# pip install vaderSentiment


STOPWORDS = dict(zip(stopwords.words('english'), [''] * len(stopwords.words('english'))))


def get_sent(vs):
    vs.pop('compound')
    v=vs.values()
    k=vs.keys()
    return k[v.index(max(v))]

def get_sentiment(sentence):
    words = re.sub('[^a-z0-9 ]', ' ', sentence.lower())
    words = re.sub(' +', ' ', words)
    words = ' '.join([word for word in words.split(' ') if word not in STOPWORDS])
    return get_sent(vaderSentiment(words))


# '/Users/cchen224/Downloads/expended_thread_aws/*.txt'
def sentiment(dir_in, fp_out):
    files = glob(dir_in)
    bar = ProgressBar(total=len(files))
    for file in files:
        bar.move().log()
        with open(file, 'r') as i:
            csvreader = csv.reader(i, delimiter='\t')
            with open(fp_out, 'a') as o:
                csvwriter = csv.writer(o)
                is_write = True
                for row in csvreader:
                    if is_write:
                        tid = row[1]
                        uid = row[2]
                        sentence = row[3].strip()
                        words = re.sub('[^a-z0-9 ]', ' ', sentence.lower())
                        words = re.sub(' +', ' ', words)
                        words = ' '.join([word for word in words.split(' ') if word not in STOPWORDS])
                        csvwriter.writerow([tid, uid, sentence, get_sent(vaderSentiment(words))])
                    is_write = row == []
    bar.close()

# sentiment('/Users/cchen224/Downloads/expended_thread_aws/*.txt', '/Users/cchen224/Downloads/sentiments.csv')


import re


def parse_media(item):
    photo_caption = re.sub('\n|"', '', item.caption.text.encode('utf-8')) if item.caption is not None else ''
    photo_comment = item.comment_count
    created_time = item.created_time
    photo_filter = item.filter
    photo_id = item.id
    photo_url = item.images.get('standard_resolution').url
    photo_like = item.like_count
    photo_link = item.link
    tags = item.tags if hasattr(item, 'tag') else ''
    item_type = item.type
    user_username = item.user.username.encode('utf-8')
    user_id = item.user.id
    photo_userliked = item.user_has_liked

    users_in_photo = item.users_in_photo
    usernames = '|'.join([user.user.id for user in users_in_photo])
    no_users_in_photo = len(users_in_photo)
    # if len(users_in_photo) > 0:
    #     no_users_in_photo = len(users_in_photo)
    #
    # else:
    #     no_users_in_photo = 0
    #     usernames = ''

    if hasattr(item, 'location'):
        location_disclosed = 1
        photo_location_id = item.location.id
        # print item.location.point
        photo_location_name = item.location.name.encode('utf-8')
        if item.location.point is None:
            photo_location_lat = ''
            photo_location_long = ''
        else:
            photo_location_lat = item.location.point.latitude
            photo_location_long = item.location.point.longitude
    else:
        location_disclosed = 0
        photo_location_id = ''
        photo_location_name = ''
        photo_location_lat = ''
        photo_location_long = ''


    return [user_id,
            user_username,
            photo_id,
            created_time,
            photo_caption,
            photo_filter,
            photo_link,
            photo_url,
            item_type,
            photo_like,
            photo_comment,
            photo_userliked,
            tags,
            no_users_in_photo,
            usernames,
            location_disclosed,
            photo_location_id,
            photo_location_name,
            photo_location_lat,
            photo_location_long]



import re


def parse_user_info(user):
    userid = user.id
    username = user.username.encode('utf-8')#.encode('ascii', 'ignore')
    NoFollower = user.counts.get('followed_by')
    NoFriends = user.counts.get('follows')
    NoMedia = user.counts.get('media')
    userWebsite = user.website.encode('utf-8')#.encode('ascii', 'ignore')
    userBio = user.bio.encode('utf-8')#.encode('ascii', 'ignore')
    userBio = re.sub('\n|"', ' ', userBio)
    userBio = re.sub(' +', ' ', userBio)
    userProfilePic = user.profile_picture.encode('utf-8')#.encode('ascii', 'ignore')

    return [userid,
            username,
            NoFollower,
            NoFriends,
            NoMedia,
            userWebsite,
            userBio,
            userProfilePic]



import random
import re
import sys
import time
import traceback
from os import listdir

from cutils import Gmail
from cutils import ProgressBar

from webparser_tweet import parse_tweet


# pip install --upgrade google-api-python-client
# pip install beautifulsoup4

def crawl(fp, outputFile):
    gmail = Gmail(minDuration=0).set_from_to().set_subject('AWS TweetContent Status')

    total = 0
    f_scan = open(fp, 'r')
    for _ in f_scan:
        total += 1
    f_scan.close()
    bar = ProgressBar(total=total)

    print fp, '...'
    f = open(fp, 'r')
    o = open(outputFile, 'a')
    i = 0
    for line in f:
        tokens = line.split(',')
        tid = tokens[0]
        uid = tokens[1]
        screen_name = re.sub('\'', '"', tokens[3])
        time_date = re.sub('\'', '"', tokens[2])
        try:
            i += 1
            if i % 10000 == 0:
                print i, 'sleeping for a sec.'
                time.sleep(random.uniform(4, 9))

            info = parse_tweet(uid, tid)
            if info['has_result'] is True:
                o.write(','.join([uid, screen_name.strip(), tid, time_date, info['text'],
                                  info['hashtags'], info['cashtags'], info['mentions'], info['emojis'],
                                  info['retweets'], info['likes'],
                                  info['n_uid'], info['n_tid']]))
                o.write('\n')
            elif info['has_result'] == 'ConnectionError':
                gmail = gmail.send_message(fp + '\n' + uid + ' ' + tid + ' Connection Error.')
                break
        except:
            gmail = gmail.send_message(fp + '\n' + uid + ' ' + tid + '\n' + traceback.format_exc())

        bar.move()
        bar.log()
    f.close()
    o.close()
    bar.close()


if __name__ == '__main__':
    argvs = sys.argv[1:]
    target_folder = 'docs/'
    docs = listdir(target_folder)
    gmail = Gmail().set_from_to().set_subject('AWS TweetContent Status')
    for doc in docs:
        if doc.endswith('.sample') and doc.startswith(argvs[0]):
            crawl(target_folder + doc,
                  target_folder + re.sub('sample', 'csv', doc))
            gmail = gmail.send_message(doc + ' Done!')
    gmail = gmail.send_message('\nYear' + argvs[0] + ' Done!')
    gmail = gmail.close()



from bs4 import BeautifulSoup

from webparser_error import *
from webparser_utility import *


def parse_tweet(tiduid):
    """
    :param tiduid: tid@uid
    :return: uid, screen_name, tid, timestamp, text, hashtags, cashtags, mentions, emojis, retweets, likes
    """
    tokens = tiduid.split('@')
    tid = tokens[0]
    uid = tokens[1]
    url = 'https://twitter.com/' + uid + '/status/' + tid
    html = get_html(url)

    if html is None:
        return False

    soup = BeautifulSoup(html, 'html.parser')
    for fnc in [is_account_not_suspended(soup), is_content_not_protected(soup), is_page_exists(soup)]:
        if not fnc:
            return []

    # PermalinkOverlay
    cardwrap = soup.find('div', class_='permalink-inner permalink-tweet-container')

    properties = cardwrap.find('div', class_=re.compile('^permalink-tweet')).attrs
    tid = properties['data-item-id']
    uid = properties['data-user-id']
    screen_name = properties['data-screen-name']

    out = [uid, screen_name, tid]
    out.extend(parse_header(cardwrap.find('div', class_='permalink-header')))
    out.extend(parse_text(cardwrap.find('div', class_="js-tweet-text-container")))
    out.extend(parse_fixer(cardwrap.find('div', class_='js-tweet-details-fixer tweet-details-fixer')))

    return out
